#include <iostream>
using namespace std;

#include <Date.h>

const Date::date_type Date::dys[]={31,28,31,30,31,30,31,31,30,31,30,31};

void Date::SetDate(date_type mo, date_type da, date_type yr)
{
    if (mo < 1 || mo > 12 || yr < 1)  {
        ndays = 0;  // invalid month or year or null date
        return;
    }
    // compute days thru last year
    ndays = (yr-1) * 365 + (yr-1) / 4 - (yr-1) / 1000;
    for (short int i = 0; i < mo; i++)  {
        int dy = dys[i];
        if (i == 1 && IsLeapYear(yr))
            dy++;          // make Feb's days 29 if leap year
        if (i < mo-1)      // add in all but this month's days
            ndays += dy;
        else if (da > dy)  {
            ndays = 0;     // invalid day
            return;
        }
    }
    ndays += da;  // add in this month's days
}

void Date::GetDate(date_type& mo, date_type& da, date_type& yr) const
{
    da = ndays;
    if (ndays == 0)  {
        yr = mo = 0;
        return;
    }
    for (yr = 1;; yr++)  {
        int daysthisyear = IsLeapYear(yr) ? 366 : 365;
        if (da <= daysthisyear)
            break;
        da -= daysthisyear;
    }
    for (mo = 1; mo < 13; mo++)	{
        int dy = dys[mo-1];
        if (mo == 2 && IsLeapYear(yr))
            dy++;   // make Feb's days 29 if leap year
        if (da <= dy)
            break;
        da -= dy;
    }
}
Date::date_type Date::GetMonth() const
{
    date_type mo, da, yr;
    GetDate(mo, da, yr);
    return mo;
}
Date::date_type Date::GetDay()   const
{
    date_type mo, da, yr;
    GetDate(mo, da, yr);
    return da;
}
Date::date_type Date::GetYear()   const
{
    date_type mo, da, yr;
    GetDate(mo, da, yr);
    return yr;
}

std::ostream& operator<<(std::ostream& os, const Date& dt)
{
    os << dt.GetMonth() << '/'
       << dt.GetDay()   << '/'
       << dt.GetYear();
    return os;
}
std::istream& operator>>(std::istream& is, Date& dt)
{
    Date::date_type mo, da, yr;
    is >> mo >> da >> yr;
    dt.SetDate(mo, da, yr);
    return is;
}

ptl::PTLStore& operator<<(ptl::PTLStore& os, const Date& dt)
{
    os << dt.GetMonth()
       << dt.GetDay()
       << dt.GetYear();
    return os;
}
ptl::PTLStore& operator>>(ptl::PTLStore& is, Date& dt)
{
    Date::date_type mo, da, yr;
    is >> mo >> da >> yr;
    dt.SetDate(mo, da, yr);
    return is;
}
