#ifndef DATE_H
#define DATE_H

#include <iostream>
#include <ptl.h>
namespace ptl = PersistentTemplateLibrary;

class Date  {
public:
	typedef unsigned int date_type;
protected:
    static const date_type dys[];
    long int ndays; // days inclusive since Jan 1,1 (1/1/1 == 1)
public:
    // --- default and initializing constructor
    Date(date_type mo = 0, date_type da = 0, date_type yr = 0)
        { SetDate(mo, da, yr); }
    // --- copy constructor
    Date(const Date& dt)
        { *this = dt; }
    // --- destructor
    virtual ~Date() {}
    // --- overloaded assignment operator
    Date& operator=(const Date& dt)
        { ndays = dt.ndays; return *this; }
    // --- overloaded arithmetic operators
    Date  operator+(int n) const
        { Date dt(*this); dt += n; return dt; }
    Date  operator-(int n) const
        { Date dt(*this); dt -= n; return dt; }
    Date& operator+=(int n)
        { ndays += n; return *this; }
    Date& operator-=(int n)
        { ndays -= n; return *this; }
    Date& operator++()	   // prefix
        { ++ndays; return *this; }
    Date  operator++(int)  // postfix
        { Date dt(*this); dt.ndays++; return dt; }
    Date& operator--()	   // prefix
        { --ndays; return *this; }
    Date  operator--(int)  // postfix
        { Date dt(*this); dt.ndays--; return dt; }
    long int operator-(const Date& dt) const
        { return ndays-dt.ndays; }
    // --- overloaded relational operators
    bool operator==(const Date& dt) const
        { return ndays == dt.ndays; }
    bool operator!=(const Date& dt) const
        { return ndays != dt.ndays; }
    bool operator< (const Date& dt) const
        { return ndays <  dt.ndays; }
    bool operator> (const Date& dt) const
        { return ndays >  dt.ndays; }
    bool operator<=(const Date& dt) const
        { return ndays <= dt.ndays; }
    bool operator>=(const Date& dt) const
        { return ndays >= dt.ndays; }
    // --- getter and setter functions
    void SetDate(date_type mo, date_type da, date_type yr);
    void SetMonth(date_type mo)
        { *this = Date(mo, GetDay(), GetYear()); }
    void SetDay(date_type da)
        { *this = Date(GetMonth(), da, GetYear()); }
    void SetYear(date_type yr)
        { *this = Date(GetMonth(), GetDay(), yr); }
    void GetDate(date_type& mo, date_type& da, date_type& yr) const;
    date_type GetMonth() const;
    date_type GetDay()   const;
    date_type GetYear()  const;
    // --- test for leap year
    bool IsLeapYear(date_type yr) const
        { return ((yr % 4)==0 && (yr % 1000)!=0); }
    bool IsLeapYear() const
        { return IsLeapYear(GetYear()); }
};
// ----- overloaded (int + Date)
inline Date operator+(int n, const Date& dt)
{
    return dt + n;
}

std::ostream& operator<<(std::ostream& os, const Date& dt);
std::istream& operator>>(std::istream& is, Date& dt);
ptl::PTLStore& operator<<(ptl::PTLStore& os, const Date& dt);
ptl::PTLStore& operator>>(ptl::PTLStore& is, Date& dt);

#endif
