;--------------------------------------------------------------------;
; Author: Dale Roberts
; Module Name: inthack.asm
;
;  The Pentium PCI chipset interrupt edge/level control registers can
; be used to generate interrupts.  To do this, the bit is set high, to
; the LEVEL setting.  This causes the interrupt to be generated, but
; the interrupt remains asserted even after the 80x86 acknowledges it.
; To get around this and make our interrupt look like an "edge"
; triggered interrupt, the IDT (Interrupt Descriptor Table) entries
; for the interrupts that DOIRQ uses are hooked to point directly to
; this assembler routine.  The routine determines which interrupt
; caused it to be invoked, and resets the corresponding bit in the
; edge/level registers.
;
;  The 486 Pentium chipset handles things differently.  To generate an
; interrupt, you set the edge/level bit first high, then to low (edge
; mode).  Then, the interrupt is generated in edge mode, and so the
; bit gets automatically reset when it is acknowledged, and this
; assembler routine is unnecessary.
;
;--------------------------------------------------------------------;

        .386

        .model  small,c

        assume ds:FLAT,es:FLAT,ss:FLAT
        assume fs:nothing,gs:nothing

	.data
	OldIRQHandler dd 0

	extrn OldIRQTable:dword
	public asmIntHandler

        .code

;*********************************************************************
;  Table of interrupt handlers.  Each mini-handler does a CALL to the
; actual service routine.  The main service routine uses the return
; address on the stack to determine which mini-handler called it, and
; thus determines which interrupt caused it to be called.
;*********************************************************************

	asmIRQ0Handler@0 proc near
	public asmIRQ0Handler@0
	call asmIntHandler
	asmIRQ0Handler@0 endp

	asmIRQ1Handler@0 proc near
	public asmIRQ1Handler@0
	call asmIntHandler
	asmIRQ1Handler@0 endp

	asmIRQ2Handler@0 proc near
	public asmIRQ2Handler@0
	call asmIntHandler
	asmIRQ2Handler@0 endp

	asmIRQ3Handler@0 proc near
	public asmIRQ3Handler@0
	call asmIntHandler
	asmIRQ3Handler@0 endp

	asmIRQ4Handler@0 proc near
	public asmIRQ4Handler@0
	call asmIntHandler
	asmIRQ4Handler@0 endp

	asmIRQ5Handler@0 proc near
	public asmIRQ5Handler@0
	call asmIntHandler
	asmIRQ5Handler@0 endp

	asmIRQ6Handler@0 proc near
	public asmIRQ6Handler@0
	call asmIntHandler
	asmIRQ6Handler@0 endp

	asmIRQ7Handler@0 proc near
	public asmIRQ7Handler@0
	call asmIntHandler
	asmIRQ7Handler@0 endp

	asmIRQ8Handler@0 proc near
	public asmIRQ8Handler@0
	call asmIntHandler
	asmIRQ8Handler@0 endp

	asmIRQ9Handler@0 proc near
	public asmIRQ9Handler@0
	call asmIntHandler
	asmIRQ9Handler@0 endp

	asmIRQ10Handler@0 proc near
	public asmIRQ10Handler@0
	call asmIntHandler
	asmIRQ10Handler@0 endp

	asmIRQ11Handler@0 proc near
	public asmIRQ11Handler@0
	call asmIntHandler
	asmIRQ11Handler@0 endp

	asmIRQ12Handler@0 proc near
	public asmIRQ12Handler@0
	call asmIntHandler
	asmIRQ12Handler@0 endp

	asmIRQ13Handler@0 proc near
	public asmIRQ13Handler@0
	call asmIntHandler
	asmIRQ13Handler@0 endp

	asmIRQ14Handler@0 proc near
	public asmIRQ14Handler@0
	call asmIntHandler
	asmIRQ14Handler@0 endp

	asmIRQ15Handler@0 proc near
	public asmIRQ15Handler@0
	call asmIntHandler
	asmIRQ15Handler@0 endp

;*********************************************************************
;  LEVEL interrupt handler.  Clears the LEVEL bit in the edge/level
; register for the interrupt that caused us to be called.  This
; makes the interrupt appear as an edge interrupt to NT.  This code
; is called directly as a result of the interrupt, before NT gets
; control.  We then jump to the original address in the IDT.
;*********************************************************************

	asmIntHandler proc near
	public asmIntHandler
;
; Save "return" address (to which we never return)
; and use it to figure out which IRQ called us.
; Can't pop directly into EAX because we must preserve
; all registers.
;
	pop OldIRQHandler
	pushad
;
; Get IRQ number into EAX: Take return address to the mini-handler,
; subtract out the address of IRQ1 Handler (the return address points
; *after* the call instruction, so we subtract out the IRQ1 call
; address to get a 0 when IRQ0Handler calls us), then divide by the
; size of the call instruction.
;
	mov eax, OldIRQHandler
	sub eax, asmIRQ1Handler@0
	mov ebx, (asmIRQ1Handler@0-asmIRQ0Handler@0)
	xor edx, edx
	div ebx
;
; Get old interrupt vector
;
	mov ebx, OldIRQTable[eax*4]
	mov OldIRQHandler, ebx
;
; Make a bit mask from the vector number
;
	mov ebx,1
	mov ecx,eax
	shl ebx,cl
	not ebx
;
; Turn off the corresponding bit in the edge/level register.
;
	mov dx,4d0h
	in al,dx
	and al,bl
	out dx,al
	inc dx
	shr bx,8
	in al,dx
	and al,bl
	out dx,al

	popad

;
;  Jump to original IRQ handler
;
	push OldIRQHandler
	ret

	asmIntHandler endp

        end

