/*********************************************************************
Author:       Dale Roberts
File:         INTINFO.H

Purpose:      Common information for DOIRQ, GENIRQ, and CHECKIRQ.
*********************************************************************/

//
//  Interrupt controller I/O port addresses.
//
#define I8259_1_CMD		0x20
#define I8259_1_MASK	0x21
#define I8259_1_EDGELVL	0x4d0

#define I8259_2_CMD		0xa0
#define I8259_2_MASK	0xa1
#define I8259_2_EDGELVL	0x4d1

//
// Bitmask of IRQs that we should never use.  This includes
// IRQ0 (timer), IRQ1 (keyboard), IRQ2 (cascade to 2nd 8259),
// IRQ6 (floppy), IRQ8 (realtime clock) and IRQ13 (coprocessor).
//
#define OFF_LIMITS_IRQ_MASK		(~0x2147)

//
//  This is the I/O Control Code used to map the buffer into the
// application's address space.
//
#define FILE_DEVICE_TESTDRV	0x8002

#define TESTDRV_CODE(c)      CTL_CODE(FILE_DEVICE_TESTDRV, \
                                    0x800 | (c), \
                                    METHOD_BUFFERED, \
                                    FILE_ANY_ACCESS)

#define IOCTL_TESTDRV_GO      TESTDRV_CODE(1)


// Number of interrupts we can queue for tracing.
#define INT_GEN_TABLE_SIZE	20
#define NUM_TRACE_ENTRIES	100
#define TRACE_MSG_STRING_SZ	32

// Make sure structures are aligned correctly.
#pragma pack(push,4)

//
//  Structure of a trace entry.
//
typedef struct {
	unsigned short int	IRR, ISR, IMR;	// The 3 8259 registers
	char				IFlag;			// Processor's interrupt flag
	char				IntNum;			// IRQ Number
	_int64				TimeStamp;		// RDTSC value
	unsigned short int	GenIRQMask;		// IRQ's generated
	char				msg[TRACE_MSG_STRING_SZ];
} INT_TRACE;

//
// Structure for information shared between DOIRQ kernel
// mode driver and GENIRQ console application.  The
// structure is mapped into shared memory so that the
// application can get live updates from the kernel mode
// interrupt routine.
//
typedef struct {
	int		CPUTicksPerUSec;	// conversion from RDTSC to uSec
	int		EnabledIRQMask;		// Bitmask to remember which
								// interrupts we are allowed to use.

	int		SerPort;			// Set to non-zero to handle
	int		SerIrq;				//  serial interrupts.

	int		CurSerIRQInterval;	// Serial IRQ statistics
	int		MaxSerIRQInterval;
	int		NumDelayedInts;		// # interrupts > 400uSec delayed
	int		NumDelayedInts2;	// # interrupts > 1000uSec delayed
	_int64	LastSerTimestamp;

								// IRQ masks to generate interrupts
	int		IntGenTable[INT_GEN_TABLE_SIZE];
	int		IntGenIdx;

								// Trace messages from interrupt
								//  routine
	INT_TRACE Trace[NUM_TRACE_ENTRIES];
	int		TraceIdx;
} INT_INFO;

#pragma pack(pop)

