
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <rpcsvc/rstat.h>
#include <sys/param.h>

#include "SojournerNode.h"

#define LOW 0
#define MEDIUM 1
#define HIGH 2

int determine_instant_threshold(char *);

int determine_smooth_threshold(int, int, int, int *);
/*                             instant_threshold, old_instant_threshold, 
                               old_threshold, consecutive_change_cnt */  
                                
void call_rstat(char *, float *);
/*              hostname, cpu_level                    */

/* Global data */

float cpu_limit = 200.0;

int current_threshold;

JNIEXPORT void JNICALL Java_SojournerNode_runNode
   (JNIEnv *env, jobject obj)
{

    char hostname[256];
    int old_threshold, instant_threshold, 
        old_instant_threshold, threshold_count;
    int loop_cnt;
    jclass clazz;
    jfieldID fid;
    threshold_count = 0;
    gethostname(hostname, sizeof(hostname));
    clazz = (*env)->GetObjectClass(env, obj);
    fid = (*env)->GetFieldID(env, clazz, "machineLoadLevel", "I");
    current_threshold = (*env)->GetIntField(env, obj, fid);
    while(1)
      {
        sleep(6);
        old_threshold = current_threshold;
        old_instant_threshold = instant_threshold;
        instant_threshold = determine_instant_threshold(hostname);
        current_threshold = 
                 determine_smooth_threshold(instant_threshold,
                                            old_instant_threshold, 
                                            old_threshold,
                                            &threshold_count);
        if (current_threshold != old_threshold)
		  (*env)->SetIntField(env, obj, fid, current_threshold);
      }

}

int determine_instant_threshold(char *hostname)
{
    float new_cpu_level;
    call_rstat(hostname, &new_cpu_level);
    if ((new_cpu_level < (cpu_limit / 2.0))) return LOW;
    if (new_cpu_level < cpu_limit)           return MEDIUM;
    return HIGH;
}  

int determine_smooth_threshold(int instant_threshold, 
                               int old_instant_threshold,
                               int old_smooth_threshold,
                               int *threshold_change_count)
{
    if (instant_threshold == old_smooth_threshold) 
      {
       *threshold_change_count = 0;
       return instant_threshold;
      }
    if (instant_threshold  == old_instant_threshold) 
                                              (*threshold_change_count)++;
    else {
           if (old_smooth_threshold == MEDIUM && 
                    (((instant_threshold == LOW) &&
                          (old_instant_threshold == HIGH)) ||
                     ((instant_threshold == HIGH) &&
                          (old_instant_threshold == LOW))))
             {
              *threshold_change_count = 1;
              return MEDIUM;
             }
           else *threshold_change_count++;
	 }

     if (instant_threshold > old_smooth_threshold)
       {
        if (*threshold_change_count > 1)
          {
           *threshold_change_count = 0;
           return ++old_smooth_threshold;
          }
        else return old_smooth_threshold;
       }

     if (*threshold_change_count > 10) 
       {
        *threshold_change_count = 0;
        return --old_smooth_threshold;                    
       }
     return old_smooth_threshold;
} 


void call_rstat(char *hostname, float * cpu_queue_length)
{

   struct statstime rstat_output;
   rstat(hostname, &rstat_output);
   *cpu_queue_length = rstat_output.avenrun[0];
}






