
import java.util.*;
import COM.objectspace.voyager.*;

public class SojournerNodeControl extends Thread 
                                  implements SojournerHost {

    SojournerLocalRegistry registry;
    SojournerNode          localNode;
    Vector                 guests;
    String                 masterSojournerServer;
    boolean                 stopNow;
    int                    currentLoadLevel, previousLoadLevel;

    public SojournerNodeControl(String masterServer)
        throws java.net.UnknownHostException, VoyagerException
    {
        masterSojournerServer = masterServer;
        registry  = new SojournerLocalRegistry(masterServer);
        guests    = new Vector();
	stopNow = false;
        localNode = new SojournerNode();
        localNode.setDaemon(true);
        localNode.start();
        currentLoadLevel = localNode.getMachineLoadLevel();
    }

    public boolean receiveGuest(VObject newGuest)
    {
       if (currentLoadLevel != SojournerNode.LOW) return false;
       guests.addElement(newGuest);
       return true;
    }

    public void evictGuests()
       throws COM.objectspace.voyager.VoyagerException
    {
       Vector newHosts = new Vector();
       Enumeration newHostEnumeration = registry.getRegisteredMachines();
       while (newHostEnumeration.hasMoreElements())
          newHosts.addElement(newHostEnumeration.nextElement());
       if (newHosts.size() == 0) 
		newHosts.addElement(masterSojournerServer + ":7777");
       for (int i = 0; i < guests.size(); i++)
       {
           String targetMachineName = 
               (String) newHosts.elementAt(i % newHosts.size());
           SojournerHost targetMachine =
               (SojournerHost) 
                  VObject.forObjectAt(targetMachineName + 
                                               "/sojournerNodeControl");
           if (targetMachine.receiveGuest((VObject) guests.elementAt(i))) 
           {    
		((VObject) guests.elementAt(i)).moveTo(targetMachineName);
                continue;
	   }
           newHosts.removeElementAt(i % newHosts.size());
           i--;
           if (newHosts.size() == 0) 
                      newHosts.addElement(masterSojournerServer + ":7777");
       }
       guests = new Vector();
    }

    public void run()
    {
        while(true)
        {
	    if (stopNow) return;
            previousLoadLevel = currentLoadLevel;
            currentLoadLevel  = localNode.getMachineLoadLevel();
            switch (currentLoadLevel)
	    {
		case SojournerNode.HIGH:   processHigh();
					   break;
		case SojournerNode.MEDIUM: processMedium();
                                           break;
                case SojournerNode.LOW:    processLow();
	    }
        } 
    }


    public void stopNode()
        throws VoyagerException
    {
	stopNow = true;                         // Stop load balancing
	registry.unregister();                  // Don't invite new work
	currentLoadLevel = SojournerNode.HIGH;  // Don't accept new work
	evictGuests();                          // Get rid of current jobs
        localNode = null;
    }


    void processHigh()
    {
        if (previousLoadLevel != SojournerNode.HIGH)
        {   
             try
	     {        
                  evictGuests();
                  if (previousLoadLevel == SojournerNode.LOW) 
                                                  registry.unregister();
             }
             catch (VoyagerException e) { }
        }
	try
        {
            Thread.sleep(300000);
        }
	catch(InterruptedException e) { }
    }

    void processMedium()
    {   
        if (previousLoadLevel == SojournerNode.LOW) 
        {    
	     try
             {
                  registry.unregister();
             }
             catch(VoyagerException e) { }
        }
	try
        {
            Thread.sleep(6000);
        }
	catch(InterruptedException e) { }
    }

    void processLow()
    {   
        if (previousLoadLevel != SojournerNode.LOW)
        {    
	     try
             {
                  registry.register();
             }
             catch(VoyagerException e) { }
        }
	try
        {
            Thread.sleep(6000);
        }
	catch(InterruptedException e) { }
    }
}



