// --------- ptlstore.h
#ifndef PTLSTORE_H
#define PTLSTORE_H

#include <fstream>

namespace PersistentTemplateLibrary	{

// ========================================================
//                  PTLStore class
//  class to encapsulate container store disk file
// ========================================================
class PTLStore {
    std::string name;
    bool isreading;
    int objcount;
public:
    std::ifstream ifile;
    std::ofstream ofile;
    PTLStore() : isreading(false), objcount(0)
	    { }
    void open(const std::string& nm);
    void open(int ct);
    void close();
    int count() const
	    { return objcount; }
};
// ========================================================
// overloaded insertion and extraction 
// operator template functions
// to manage disk I/O of intrinsic types
// and non-polymorphic user-defined types
// that have no pointer or reference data members
// ========================================================
template <class T>
PTLStore& operator<<(PTLStore& cs, const T& t)
{
    cs.ofile.write((const char*)&t, sizeof t);
    return cs;
}
template <class T>
PTLStore& operator>>(PTLStore& cs, T& t)
{
    cs.ifile.read((char*)&t, sizeof t);
    return cs;
}
// ========================================================
// overloaded insertion and extraction operators
// to manage disk I/O of Standard C++ string objects
// ========================================================
PTLStore& operator<<(PTLStore& cs, const std::string& str);
PTLStore& operator>>(PTLStore& cs, std::string& str);

} // namespace PersistentTemplateLibrary

#endif // PTLSTORE_H
